import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import Context from '../../shared/Context';
import { isTrue, registerElement, makeUniqueId, validateDOMAttributes, processChildren, extendPropsWithContext } from '../../shared/component-helper';
import { spacingPropTypes, createSpacingClasses } from '../space/SpacingHelper';
import Icon from '../icon/Icon';
import GlobalStatusProvider from '../global-status/GlobalStatusProvider';
import { skeletonDOMAttributes, createSkeletonClass } from '../skeleton/SkeletonHelper';
export default class FormStatus extends React.PureComponent {
  static enableWebComponent() {
    registerElement(FormStatus.tagName, FormStatus, FormStatus.defaultProps);
  }

  static getContent(props) {
    if (props.text) {
      if (isTrue(props.text)) {
        return null;
      }

      return props.text;
    }

    return processChildren(props);
  }

  static getIcon({
    state,
    icon,
    icon_size
  }) {
    if (typeof icon === 'string') {
      let IconToLoad = icon;

      switch (state) {
        case 'info':
        case 'information':
          IconToLoad = InfoIcon;
          break;

        case 'warn':
        case 'warning':
          IconToLoad = WarnIcon;
          break;

        case 'error':
        default:
          IconToLoad = ErrorIcon;
      }

      icon = React.createElement(Icon, {
        icon: React.createElement(IconToLoad, {
          title: null
        }),
        size: icon_size,
        inherit_color: false
      });
    }

    return icon;
  }

  static getDerivedStateFromProps(props, state) {
    if (state._id !== props.id) {
      state.id = props.id;
    }

    state._id = props.id;
    return state;
  }

  constructor(props) {
    super(props);
    this.state = {
      id: null
    };

    this.init = () => {
      if (this._isMounted) {
        if (this.gsProvider) {
          this.gsProvider.isReady();
        }

        this.updateWidth();
      }
    };

    this.state.id = props.id || makeUniqueId();

    if (props.status !== 'info') {
      this.gsProvider = GlobalStatusProvider.init(props.global_status_id || 'main', provider => {
        const {
          state,
          text,
          label
        } = this.props;
        provider.add({
          state,
          status_id: `${this.state.id}-gs`,
          item: {
            status_id: this.state.id,
            text,
            status_anchor_label: label,
            status_anchor_url: true
          }
        });
      });
    }

    this._ref = React.createRef();
  }

  componentDidMount() {
    this._isMounted = true;

    if (document.readyState === 'complete') {
      this.init();
    } else if (typeof window !== 'undefined') {
      window.addEventListener('load', this.init);
    }
  }

  componentWillUnmount() {
    this._isMounted = false;

    if (this.gsProvider) {
      const status_id = `${this.state.id}-gs`;
      this.gsProvider.remove(status_id);
    }

    if (typeof window !== 'undefined') {
      window.removeEventListener('load', this.init);
    }
  }

  componentDidUpdate(prevProps) {
    if (this.gsProvider && (prevProps.text !== this.props.text || prevProps.state !== this.props.state)) {
      const {
        state,
        text,
        label
      } = this.props;
      const status_id = `${this.state.id}-gs`;
      this.gsProvider.update(status_id, {
        state,
        item: {
          status_id: this.state.id,
          text,
          status_anchor_label: label,
          status_anchor_url: true
        }
      }, {
        preventRestack: true
      });
    }

    this.updateWidth();
  }

  correctStatus(state) {
    switch (state) {
      case 'information':
        state = 'info';
        break;

      case 'warning':
        state = 'warn';
        break;
    }

    return state;
  }

  updateWidth() {
    if (this._ref.current) {
      const {
        width_element,
        width_selector
      } = this.props;
      setMaxWidthToElement({
        element: this._ref.current,
        widthElement: width_element && width_element.current,
        widthSelector: width_selector
      });
    }
  }

  render() {
    const props = extendPropsWithContext(this.props, FormStatus.defaultProps, {
      skeleton: this.context && this.context.skeleton
    }, this.context.FormRow, this.context.FormStatus);

    const {
      title,
      status: rawStatus,
      state: rawState,
      size,
      variant,
      hidden,
      className,
      animation,
      class: _className,
      text_id,
      icon,
      icon_size,
      skeleton
    } = props,
          attributes = _objectWithoutPropertiesLoose(props, ["title", "status", "state", "size", "variant", "hidden", "className", "animation", "class", "text_id", "status_id", "id", "text", "icon", "icon_size", "skeleton", "children"]);

    const state = this.correctStatus(rawStatus || rawState);
    const iconToRender = FormStatus.getIcon({
      state,
      icon,
      icon_size
    });
    const contentToRender = FormStatus.getContent(this.props);

    if (contentToRender === null) {
      return React.createElement(React.Fragment, null);
    }

    const hasStringContent = typeof contentToRender === 'string' && contentToRender.length > 0;

    const params = _extends({
      id: this.state.id,
      hidden,
      className: classnames(`dnb-form-status dnb-form-status--${state} dnb-form-status__size--${size}`, createSpacingClasses(props), className, _className, variant && `dnb-form-status__variant--${variant}`, animation && `dnb-form-status__animation--${animation}`, hasStringContent && 'dnb-form-status--has-content'),
      title
    }, attributes);

    const textParams = {
      className: classnames('dnb-form-status__text', createSkeletonClass('font', skeleton, this.context)),
      id: text_id
    };

    if (hidden) {
      params['aria-hidden'] = hidden;
    }

    skeletonDOMAttributes(params, skeleton, this.context);
    validateDOMAttributes(this.props, params);
    validateDOMAttributes(null, textParams);
    return React.createElement("span", _extends({}, params, {
      ref: this._ref
    }), React.createElement("span", {
      className: "dnb-form-status__shell"
    }, iconToRender, React.createElement("span", textParams, contentToRender)));
  }

}
FormStatus.tagName = 'dnb-form-status';
FormStatus.contextType = Context;
FormStatus.defaultProps = {
  id: null,
  title: null,
  text: null,
  label: null,
  icon: 'error',
  icon_size: 'medium',
  size: 'default',
  variant: null,
  state: 'error',
  status: null,
  global_status_id: null,
  hidden: false,
  text_id: null,
  width_selector: null,
  width_element: null,
  class: null,
  animation: null,
  skeleton: null,
  className: null,
  children: null
};
process.env.NODE_ENV !== "production" ? FormStatus.propTypes = _extends({
  id: PropTypes.string,
  title: PropTypes.string,
  text: PropTypes.oneOfType([PropTypes.string, PropTypes.bool, PropTypes.func, PropTypes.node]),
  label: PropTypes.node,
  icon: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  icon_size: PropTypes.string,
  state: PropTypes.oneOfType([PropTypes.bool, PropTypes.string, PropTypes.oneOf(['error', 'warn', 'info'])]),
  variant: PropTypes.oneOf(['flat', 'outlined']),
  size: PropTypes.oneOf(['default', 'large']),
  status: PropTypes.oneOfType([PropTypes.bool, PropTypes.string, PropTypes.oneOf(['error', 'warn', 'info'])]),
  global_status_id: PropTypes.string,
  hidden: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  text_id: PropTypes.string,
  width_selector: PropTypes.string,
  width_element: PropTypes.object,
  class: PropTypes.string,
  animation: PropTypes.string,
  skeleton: PropTypes.oneOfType([PropTypes.string, PropTypes.bool])
}, spacingPropTypes, {
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node])
}) : void 0;

var _ref = React.createElement("path", {
  d: "M23.625 17.864A3.547 3.547 0 0120.45 23H3.548a3.546 3.546 0 01-3.172-5.136l8.45-14.902a3.548 3.548 0 016.347 0l8.452 14.902z",
  fill: "#DC2A2A"
});

var _ref2 = React.createElement("path", {
  d: "M12 16.286a1.286 1.286 0 100 2.572 1.286 1.286 0 000-2.572z",
  fill: "#fff"
});

var _ref3 = React.createElement("path", {
  d: "M12 13.818v-5",
  stroke: "#fff",
  strokeWidth: "1.5",
  strokeLinecap: "round",
  strokeLinejoin: "round"
});

export const ErrorIcon = props => React.createElement("svg", _extends({
  width: "24",
  height: "24",
  viewBox: "0 0 24 24",
  fill: "none",
  role: "presentation"
}, props), props && props.title && React.createElement("title", null, props.title), _ref, _ref2, _ref3);
process.env.NODE_ENV !== "production" ? ErrorIcon.propTypes = {
  title: PropTypes.string
} : void 0;
ErrorIcon.defaultProps = {
  title: 'error'
};

var _ref4 = React.createElement("path", {
  d: "M23.625 17.864A3.547 3.547 0 0120.45 23H3.548a3.546 3.546 0 01-3.172-5.136l8.45-14.902a3.548 3.548 0 016.347 0l8.452 14.902z",
  fill: "#FDBB31"
});

var _ref5 = React.createElement("path", {
  d: "M12 16.286a1.286 1.286 0 100 2.572 1.286 1.286 0 000-2.572z",
  fill: "#333"
});

var _ref6 = React.createElement("path", {
  d: "M12 13.818v-5",
  stroke: "#333",
  strokeWidth: "1.5",
  strokeLinecap: "round",
  strokeLinejoin: "round"
});

export const WarnIcon = props => React.createElement("svg", _extends({
  width: "24",
  height: "24",
  viewBox: "0 0 24 24",
  fill: "none",
  role: "presentation"
}, props), props && props.title && React.createElement("title", null, props.title), _ref4, _ref5, _ref6);
process.env.NODE_ENV !== "production" ? WarnIcon.propTypes = {
  title: PropTypes.string
} : void 0;
WarnIcon.defaultProps = {
  title: 'error'
};

var _ref7 = React.createElement("path", {
  fillRule: "evenodd",
  clipRule: "evenodd",
  d: "M11.268 0a11.25 11.25 0 105.566 21.017l6.112 2.91a.75.75 0 001-1l-2.911-6.112A11.234 11.234 0 0011.268 0z",
  fill: "#007272"
});

var _ref8 = React.createElement("circle", {
  cx: "11",
  cy: "6.5",
  r: ".5",
  fill: "#fff",
  stroke: "#fff"
});

var _ref9 = React.createElement("path", {
  d: "M13.75 16H13a1.5 1.5 0 01-1.5-1.5v-3.75a.75.75 0 00-.75-.75H10",
  stroke: "#fff",
  strokeWidth: "1.5",
  strokeLinecap: "round",
  strokeLinejoin: "round"
});

export const InfoIcon = props => React.createElement("svg", _extends({
  width: "24",
  height: "24",
  viewBox: "0 0 24 24",
  fill: "none",
  role: "presentation"
}, props), props && props.title && React.createElement("title", null, props.title), _ref7, _ref8, _ref9);
process.env.NODE_ENV !== "production" ? InfoIcon.propTypes = {
  title: PropTypes.string
} : void 0;
InfoIcon.defaultProps = {
  title: 'info'
};
export function setMaxWidthToElement({
  element,
  id = null,
  widthElement = null,
  widthSelector = null
}) {
  if (!(element && typeof window !== 'undefined')) {
    return;
  }

  try {
    if (!id && !widthSelector) {
      id = element.getAttribute('id');
    }

    let width = sumElementWidth({
      widthElement,
      widthSelector: widthSelector || id.replace('-form-status', '') || id
    });

    if (width > 40) {
      const minWidth = 12 * 16;

      if (width < minWidth) {
        width = minWidth;
      }

      const remWidth = `${width / 16}rem`;
      const cS = window.getComputedStyle(element);
      const hasCustomWidth = element.style.maxWidth ? false : cS.minWidth !== '' && cS.minWidth !== 'auto' || cS.maxWidth !== '' && cS.maxWidth !== 'none';

      if (!hasCustomWidth) {
        element.style.maxWidth = remWidth;
      }
    }
  } catch (e) {}
}

function sumElementWidth({
  widthElement,
  widthSelector
}) {
  let width = 0;

  if (typeof document === 'undefined') {
    return width;
  }

  try {
    const ids = widthElement ? [widthElement] : widthSelector.split(/, |,/g);
    width = ids.reduce((acc, cur) => {
      const elem = typeof cur === 'string' ? cur[0] === '.' ? document.querySelector(cur) : document.getElementById(cur) : cur;
      let width = elem && elem.offsetWidth || window.getComputedStyle(elem).width;

      if (/em|rem/.test(width)) {
        width = parseFloat(width) * 16;
      }

      if (width > 0) {
        if (acc > 0) {
          acc += 16;
        }

        acc += width;
      }

      return acc;
    }, width);
  } catch (e) {}

  return width;
}