import { warn } from '../../shared/component-helper';
export default class Animation {
  constructor() {
    this.stack = [];
    this.events = [];
  }

  add(animation) {
    if (!animation.type) {
      warn('You should define an animation type.');
    }

    const isInProgress = this.isInProgress(animation);

    if (isInProgress) {
      if (typeof animation.onPartial === 'function') {
        animation.onPartial(isInProgress);
      }

      return animation;
    }

    this.stack.push(animation);
    return this.runNext();
  }

  unbind() {
    this.stack.forEach(animation => {
      animation.onReset = null;
      this.reset(animation);
    });
    this.stack = [];
    this.events = [];
  }

  reset(animation) {
    if (!animation) {
      return;
    }

    clearTimeout(animation._durationId);
    clearTimeout(animation._delayId);

    if (typeof animation.onReset === 'function') {
      animation.onReset(animation);
    }

    this.stack = this.stack.filter(a => a !== animation);
  }

  isInProgress(animation) {
    return this.stack.length > 0 && this.stack[0].running && animation.type === this.stack[0].type && this.stack[0];
  }

  runNext() {
    const animation = this.stack[0];

    if (!animation || animation.running) {
      return null;
    }

    animation.running = true;

    const run = () => {
      if (typeof animation.onStart === 'function') {
        animation.onStart(animation);
      }

      this.runGlobalEvents(animation, 'onStart');

      const next = () => {
        animation.running = false;

        if (typeof animation.onComplete === 'function') {
          animation.onComplete(animation);
        }

        this.runGlobalEvents(animation, 'onComplete');
        this.stack = this.stack.filter(a => a !== animation);
        this.runNext();
      };

      clearTimeout(animation._durationId);

      if (animation.duration > 0) {
        animation._durationId = setTimeout(next, animation.duration);
      } else {
        next();
      }
    };

    clearTimeout(animation._delayId);

    if (animation.delay > 0) {
      animation._delayId = setTimeout(run, animation.delay);
    } else {
      run();
    }

    return animation;
  }

  onStart(callback) {
    this.events.push({
      callback,
      type: 'onStart'
    });
  }

  onComplete(callback) {
    this.events.push({
      callback,
      type: 'onComplete'
    });
  }

  runGlobalEvents(animation, type) {
    this.events.forEach(event => {
      if (type === event.type && typeof event.callback === 'function') {
        event.callback(animation);
      }
    });
    this.events = [];
  }

}