import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import keycode from 'keycode';
import Context from '../../shared/Context';
import { warn, isTrue, AnimateHeight, makeUniqueId, registerElement, validateDOMAttributes, dispatchCustomElementEvent, convertJsxToString, processChildren, extendPropsWithContext } from '../../shared/component-helper';
import { skeletonDOMAttributes, createSkeletonClass } from '../skeleton/SkeletonHelper';
import { spacingPropTypes, createSpacingClasses } from '../space/SpacingHelper';
import GlobalStatusController, { GlobalStatusInterceptor } from './GlobalStatusController';
import GlobalStatusProvider from './GlobalStatusProvider';
import Icon from '../icon/Icon';
import Button from '../button/Button';
import Section from '../section/Section';
import { IS_IE11 } from '../../shared/helpers';
export default class GlobalStatus extends React.PureComponent {
  static enableWebComponent() {
    registerElement(GlobalStatus.tagName, GlobalStatus, GlobalStatus.defaultProps);
  }

  static getContent(props) {
    if (props.text) return props.text;
    return processChildren(props);
  }

  static getIcon({
    state,
    icon,
    icon_size
  }) {
    if (typeof icon === 'string') {
      let IconToLoad = icon;

      switch (state) {
        case 'info':
        case 'information':
          IconToLoad = InfoIcon;
          break;

        case 'error':
        default:
          IconToLoad = ErrorIcon;
      }

      icon = React.createElement(Icon, {
        icon: React.createElement(IconToLoad, {
          title: null
        }),
        size: icon_size
      });
    }

    return icon;
  }

  static getDerivedStateFromProps(props, state) {
    if (state._listenForPropChanges) {
      if (state._items !== props.items) {
        state.globalStatus = GlobalStatusProvider.combineMessages([state.globalStatus, props]);
      }
    }

    state._items = props.items;
    state._listenForPropChanges = true;
    return state;
  }

  constructor(props) {
    super(props);
    this.state = {
      globalStatus: null,
      isActive: false,
      isAnimating: false,
      keepContentVisible: false,
      _listenForPropChanges: false
    };

    this.setVisible = ({
      delay = parseFloat(this.props.delay)
    } = {}) => {
      const {
        isActive,
        initialOpened
      } = this.state;

      if (isActive === true && initialOpened) {
        if (!this.adjustHeight) {
          this.adjustHeight = this.anim.adjustFrom();
        }

        this.setState({
          keepContentVisible: true,
          _listenForPropChanges: false
        }, () => {
          this.anim.adjustTo(this.adjustHeight);
          this.adjustHeight = null;
        });
        return;
      }

      const run = () => {
        this.setState({
          isActive: true,
          initialOpened: true,
          _listenForPropChanges: false
        }, () => {
          this.anim.open();
        });
      };

      if (delay > 0) {
        clearTimeout(this._visibleTimeout);
        this._visibleTimeout = setTimeout(() => {
          run();
        }, delay);
      } else {
        run();
      }
    };

    this.setHidden = ({
      delay = parseFloat(this.props.delay)
    } = {}) => {
      const {
        isActive
      } = this.state;

      if (isActive === false) {
        return;
      }

      const run = () => {
        this.setState({
          isActive: false,
          initialOpened: false,
          _listenForPropChanges: false
        }, () => {
          this.anim.close();
        });
      };

      if (delay > 0) {
        clearTimeout(this._hiddenTimeout);
        this._hiddenTimeout = setTimeout(() => {
          run();
        }, delay);
      } else {
        run();
      }
    };

    this.onKeyDownHandler = e => {
      switch (keycode(e)) {
        case 'esc':
          e.preventDefault();
          this.closeHandler();
          break;
      }
    };

    this.closeHandler = () => {
      this.provider.add({
        status_id: 'internal-close',
        show: false
      });

      if (this.initialActiveElement) {
        try {
          this.initialActiveElement.focus();
          this.initialActiveElement = null;
        } catch (e) {
          warn(e);
        }
      }

      dispatchCustomElementEvent(this._globalStatus, 'on_hide', this._globalStatus);
    };

    this.gotoItem = (event, item) => {
      event.persist();
      const keyCode = keycode(event);

      if (item.status_id && typeof document !== 'undefined' && typeof window !== 'undefined' && keyCode === 'space' || keyCode === 'enter' || typeof keyCode === 'undefined') {
        event.preventDefault();

        try {
          const element = document.getElementById(item.status_id);

          if (!element) {
            return;
          }

          isElementVisible(element, elem => {
            try {
              elem.addEventListener('blur', e => {
                if (e.target.classList) {
                  e.target.removeAttribute('tabindex');
                }
              });
              elem.classList.add('dnb-no-focus');
              elem.setAttribute('tabindex', '-1');
              elem.focus({
                preventScroll: true
              });
            } catch (e) {
              warn(e);
            }
          });

          if (IS_IE11) {
            window.scrollTop = element.offsetTop;
          } else if (typeof element.scrollIntoView === 'function') {
            element.scrollIntoView({
              block: 'center',
              behavior: 'smooth'
            });
          }
        } catch (e) {
          warn(e);
        }
      }
    };

    this._mainRef = React.createRef();
    this._wrapperRef = React.createRef();
    this._shellRef = React.createRef();
    this.anim = new AnimateHeight({
      animate: !isTrue(props.no_animation)
    });
    this.anim.onStart(state => {
      this.setState({
        isAnimating: true,
        _listenForPropChanges: false
      });

      if (state === 'opening') {
        this.scrollToStatus();
      }
    });
    this.anim.onEnd(state => {
      this.setState({
        isAnimating: false,
        _listenForPropChanges: false
      });

      if (this.state.isActive) {
        this.setState({
          keepContentVisible: true,
          _listenForPropChanges: false
        });
        dispatchCustomElementEvent(this._globalStatus, 'on_show', this._globalStatus);

        if (state === 'opened') {
          this.setFocus();
          dispatchCustomElementEvent(this._globalStatus, 'on_open', this._globalStatus);
        }

        if (state === 'adjusted') {
          if (!isTrue(this.props.omit_set_focus_on_update)) {
            this.setFocus();
          }

          dispatchCustomElementEvent(this._globalStatus, 'on_adjust', this._globalStatus);
        }
      } else {
        this.setState({
          keepContentVisible: false,
          _listenForPropChanges: false
        });
        dispatchCustomElementEvent(this._globalStatus, 'on_close', this._globalStatus);
      }
    });
    this.provider = GlobalStatusProvider.create(props.id);
    this.state.globalStatus = this._globalStatus = this.provider.init(props);

    if (isTrue(props.show)) {
      if (isTrue(props.no_animation)) {
        this.state.isActive = true;
      }
    }

    this.provider.onUpdate(globalStatus => {
      if (globalStatus.on_close) {
        this._globalStatus = globalStatus;
      }

      this.adjustHeight = this.anim.adjustFrom();
      this.setState({
        globalStatus,
        _listenForPropChanges: false
      });
      const isActive = isTrue(globalStatus.show);

      if (isActive) {
        this.setState({
          isActive,
          _listenForPropChanges: false
        });
      }

      if (isTrue(this.props.autoclose) && this.hadContent && !this.hasContent(globalStatus) && !isTrue(this.props.show) || typeof globalStatus.show !== 'undefined' && !isTrue(globalStatus.show)) {
        this.setHidden({
          delay: 0
        });
      } else if (isTrue(this.props.show) || typeof globalStatus.show !== 'undefined' && isTrue(globalStatus.show)) {
        this.hadContent = this.hasContent(globalStatus);
        this.setVisible({
          delay: 0
        });
      }
    });
    this.initialActiveElement = null;
  }

  componentDidMount() {
    this.anim.setElem(this._shellRef.current);

    if (isTrue(this.props.show)) {
      const isActive = isTrue(this.props.show);

      if (isActive) {
        this.setVisible();
      }
    }
  }

  componentWillUnmount() {
    this.anim.remove();
    clearTimeout(this._scrollToStatusTimeout);
    clearTimeout(this._visibleTimeout);
    clearTimeout(this._hiddenTimeout);
    this.provider.empty();
  }

  componentDidUpdate(prevProps) {
    if (prevProps !== this.props) {
      const globalStatus = extendPropsWithContext(this.props, GlobalStatus.defaultProps, this.state.globalStatus);
      this.setState({
        globalStatus
      });
    }

    if (prevProps.show !== this.props.show) {
      const isActive = isTrue(this.props.show);

      if (isActive) {
        this.setVisible();
      } else {
        this.setHidden();
      }
    }
  }

  hasContent(globalStatus) {
    var _globalStatus$items;

    return ((_globalStatus$items = globalStatus.items) === null || _globalStatus$items === void 0 ? void 0 : _globalStatus$items.length) > 0 || globalStatus.text;
  }

  correctStatus(state) {
    switch (state) {
      case 'information':
        state = 'info';
        break;
    }

    return state;
  }

  setFocus() {
    if (typeof document !== 'undefined' && document.activeElement !== this._wrapperRef.current) {
      this.initialActiveElement = document.activeElement;
    }

    if (this._wrapperRef.current && !isTrue(this.props.omit_set_focus)) {
      this._wrapperRef.current.focus({
        preventScroll: true
      });
    }
  }

  scrollToStatus(isDone = null) {
    if (typeof window === 'undefined' || isTrue(this.state.globalStatus.autoscroll) === false) {
      return;
    }

    try {
      const element = this._wrapperRef.current;
      this._scrollToStatusTimeout = isElementVisible(element, isDone);

      if (element && !IS_IE11 && typeof element.scrollIntoView === 'function') {
        element.scrollIntoView({
          block: 'center',
          behavior: 'smooth'
        });
      } else {
        const top = element.offsetTop;

        if (window.scrollTo) {
          window.scrollTo({
            top,
            behavior: 'smooth'
          });
        } else {
          window.scrollTop = top;
        }
      }
    } catch (e) {
      warn('GlobalStatus: Could not scroll into view!', e);
    }
  }

  render() {
    const {
      isActive,
      isAnimating,
      keepContentVisible
    } = this.state;
    const fallbackProps = extendPropsWithContext(this.props, GlobalStatus.defaultProps, this.context.getTranslation(this.props).GlobalStatus);
    const props = extendPropsWithContext(GlobalStatusProvider.combineMessages([this.context.globalStatus, this.state.globalStatus]), GlobalStatus.defaultProps, fallbackProps);
    const lang = this.context.locale;

    const {
      title,
      state: rawState,
      className,
      no_animation,
      hide_close_button,
      close_text,
      class: _className,
      status_anchor_text,
      skeleton,
      icon,
      icon_size
    } = props,
          attributes = _objectWithoutPropertiesLoose(props, ["title", "default_title", "state", "className", "no_animation", "hide_close_button", "close_text", "class", "status_anchor_text", "skeleton", "id", "item", "items", "autoclose", "show", "delay", "autoscroll", "text", "icon", "icon_size", "children"]);

    const wrapperParams = {
      key: 'global-status',
      className: classnames("dnb-global-status__wrapper dnb-no-focus", createSkeletonClass('font', skeleton, this.context), createSpacingClasses(props), className, _className),
      'aria-live': isActive ? 'assertive' : 'off',
      onKeyDown: this.onKeyDownHandler,
      tabIndex: '-1'
    };
    const state = this.correctStatus(rawState);
    const iconToRender = GlobalStatus.getIcon({
      state,
      icon: icon || fallbackProps.icon,
      icon_size: icon_size || fallbackProps.icon_size
    });
    const titleToRender = title || fallbackProps.title || fallbackProps.default_title;
    const noAnimation = isTrue(no_animation);
    const itemsToRender = props.items || [];
    const contentToRender = GlobalStatus.getContent(props);
    const style = state === 'info' ? null : 'fire-red';

    const params = _extends({
      element: 'div',
      className: classnames(`dnb-global-status dnb-global-status--${state}`, isActive ? keepContentVisible && 'dnb-global-status--visible' : 'dnb-global-status--hidden', isAnimating && 'dnb-global-status--is-animating', noAnimation && 'dnb-global-status--no-animation')
    }, attributes);

    skeletonDOMAttributes(params, skeleton, this.context);
    validateDOMAttributes(this.props, params);
    const renderedItems = itemsToRender.length > 0 && React.createElement("ul", {
      className: "dnb-ul"
    }, itemsToRender.map((item, i) => {
      const id = item.id || item.status_id ? `${item.status_id}-${i}` : makeUniqueId();
      const text = item && item.text || item;
      const label = React.isValidElement(item.status_anchor_label) ? convertJsxToString(item.status_anchor_label) : item.status_anchor_label || '';
      const anchorText = String(item.status_anchor_text || status_anchor_text).replace('%s', label).replace(/[: ]$/g, '');
      const useAutolink = item.status_id && isTrue(item.status_anchor_url);
      return React.createElement("li", {
        key: i
      }, React.createElement("p", {
        id: id,
        className: "dnb-p"
      }, text), item && (useAutolink || item.status_anchor_url) && React.createElement("a", {
        className: "dnb-anchor",
        "aria-describedby": id,
        lang: lang,
        href: useAutolink ? `#${item.status_id}` : item.status_anchor_url,
        onClick: e => this.gotoItem(e, item),
        onKeyDown: e => this.gotoItem(e, item)
      }, anchorText));
    }));
    const hasContent = renderedItems || contentToRender;
    const renderedContent = React.createElement("div", {
      className: "dnb-global-status__content"
    }, title !== false && React.createElement("p", {
      className: "dnb-p dnb-global-status__title",
      lang: lang
    }, React.createElement("span", {
      className: "dnb-global-status__icon"
    }, iconToRender), titleToRender, !isTrue(hide_close_button) && React.createElement(CloseButton, {
      on_click: this.closeHandler,
      text: close_text,
      title: close_text
    })), hasContent && React.createElement(Section, {
      element: "div",
      style_type: "white",
      className: "dnb-global-status__message"
    }, React.createElement("div", {
      className: "dnb-global-status__message__content"
    }, typeof contentToRender === 'string' ? React.createElement("p", {
      className: "dnb-p"
    }, contentToRender) : contentToRender, renderedItems)));
    return React.createElement("div", _extends({}, wrapperParams, {
      ref: this._wrapperRef
    }), React.createElement(Section, _extends({
      style_type: style
    }, params, {
      ref: this._mainRef
    }), React.createElement("div", {
      className: "dnb-global-status__shell",
      ref: this._shellRef
    }, (isAnimating || keepContentVisible || isActive) && renderedContent)));
  }

}
GlobalStatus.tagName = 'dnb-global-status';
GlobalStatus.contextType = Context;
GlobalStatus.defaultProps = {
  id: 'main',
  status_id: 'status-main',
  title: null,
  default_title: null,
  text: null,
  items: [],
  icon: 'error',
  icon_size: 'large',
  state: 'error',
  show: null,
  autoscroll: true,
  autoclose: true,
  no_animation: false,
  close_text: 'Lukk',
  hide_close_button: false,
  omit_set_focus: false,
  omit_set_focus_on_update: true,
  delay: 0,
  status_anchor_text: null,
  skeleton: null,
  class: null,
  className: null,
  children: null,
  on_adjust: null,
  on_open: null,
  on_show: null,
  on_close: null,
  on_hide: null
};
process.env.NODE_ENV !== "production" ? GlobalStatus.propTypes = _extends({
  id: PropTypes.string,
  status_id: PropTypes.string,
  title: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  default_title: PropTypes.string,
  text: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  items: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.array]),
  icon: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  icon_size: PropTypes.string,
  state: PropTypes.oneOf(['error', 'info']),
  show: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  autoscroll: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  autoclose: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  no_animation: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  delay: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  close_text: PropTypes.string,
  hide_close_button: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  omit_set_focus: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  omit_set_focus_on_update: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  status_anchor_text: PropTypes.string,
  skeleton: PropTypes.oneOfType([PropTypes.string, PropTypes.bool])
}, spacingPropTypes, {
  class: PropTypes.string,
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  on_adjust: PropTypes.func,
  on_open: PropTypes.func,
  on_show: PropTypes.func,
  on_close: PropTypes.func,
  on_hide: PropTypes.func
}) : void 0;

const CloseButton = ({
  on_click,
  text,
  className = null
}) => React.createElement(Button, {
  type: "button",
  variant: "tertiary",
  size: "medium",
  className: classnames('dnb-global-status__close-button', className),
  icon: "close",
  icon_size: "medium",
  icon_position: "left",
  "aria-label": text,
  text: text,
  on_click: on_click
});

process.env.NODE_ENV !== "production" ? CloseButton.propTypes = {
  on_click: PropTypes.func.isRequired,
  className: PropTypes.string,
  text: PropTypes.string
} : void 0;
CloseButton.defaultProps = {
  className: null,
  text: GlobalStatus.defaultProps.close_text
};

GlobalStatus.create = (...args) => new GlobalStatusInterceptor(...args);

GlobalStatus.Set = GlobalStatus.create;
GlobalStatus.AddStatus = GlobalStatus.create;
GlobalStatus.Update = GlobalStatus.create;
GlobalStatus.Add = GlobalStatusController;
GlobalStatus.Remove = GlobalStatusController.Remove;

const isElementVisible = (elem, callback, delayFallback = 1e3) => {
  if (typeof IntersectionObserver !== 'undefined') {
    const intersectionObserver = new IntersectionObserver(entries => {
      const [entry] = entries;

      if (entry.isIntersecting) {
        intersectionObserver.unobserve(elem);

        if (typeof callback === 'function') {
          callback(elem);
        }
      }
    });
    intersectionObserver.observe(elem);
  } else {
    if (typeof callback === 'function') {
      return setTimeout(() => callback(elem), delayFallback);
    }
  }

  return null;
};

var _ref = React.createElement("path", {
  d: "M16 25a.5.5 0 100 1 .5.5 0 000-1v0",
  stroke: "#000",
  strokeWidth: "1.5",
  strokeLinecap: "round",
  strokeLinejoin: "round"
});

var _ref2 = React.createElement("path", {
  d: "M16 21V11",
  stroke: "#000",
  strokeWidth: "1.5",
  strokeLinecap: "round"
});

var _ref3 = React.createElement("path", {
  clipRule: "evenodd",
  d: "M18.161 2.347a2.408 2.408 0 00-4.322 0L1.208 28.077A2.028 2.028 0 003.029 31h25.942a2.028 2.028 0 001.821-2.923l-12.63-25.73z",
  stroke: "#000",
  strokeWidth: "1.5",
  strokeLinecap: "round",
  strokeLinejoin: "round"
});

const ErrorIcon = props => React.createElement("svg", _extends({
  width: "32",
  height: "32",
  viewBox: "0 0 32 32",
  fill: "none",
  xmlns: "http://www.w3.org/2000/svg",
  role: "presentation"
}, props), props && props.title && React.createElement("title", null, props.title), _ref, _ref2, _ref3);

process.env.NODE_ENV !== "production" ? ErrorIcon.propTypes = {
  title: PropTypes.string
} : void 0;
ErrorIcon.defaultProps = {
  title: 'error'
};

var _ref4 = React.createElement("path", {
  fillRule: "evenodd",
  clipRule: "evenodd",
  d: "M10.1 1.08A14.75 14.75 0 00.26 15.01a14.73 14.73 0 0022.16 12.74l8.27 3.94a.75.75 0 001-1l-3.94-8.27A14.75 14.75 0 0010.1 1.08zM1.76 15.01a13.25 13.25 0 1124.5 6.97.75.75 0 00-.04.72l3.2 6.73-6.72-3.2a.75.75 0 00-.72.04A13.23 13.23 0 011.76 15zM13.38 7.9a1.31 1.31 0 112.63 0 1.31 1.31 0 01-2.63 0zm-1.13 5.07c0-.41.34-.75.75-.75h1.13c1.04 0 1.88.85 1.88 1.88v5.64c0 .84.67 1.51 1.5 1.51h1.13a.75.75 0 110 1.5h-1.13a3 3 0 01-3-3V14.1c0-.2-.17-.38-.38-.38H13a.75.75 0 01-.75-.75z",
  fill: "#000"
});

const InfoIcon = props => React.createElement("svg", _extends({
  width: "32",
  height: "32",
  viewBox: "0 0 32 32",
  fill: "none",
  xmlns: "http://www.w3.org/2000/svg",
  role: "presentation"
}, props), props && props.title && React.createElement("title", null, props.title), _ref4);

process.env.NODE_ENV !== "production" ? InfoIcon.propTypes = {
  title: PropTypes.string
} : void 0;
InfoIcon.defaultProps = {
  title: 'info'
};