import _extends from "@babel/runtime/helpers/esm/extends";
import { makeUniqueId, warn } from '../../shared/component-helper';

class GlobalStatusProvider {
  static init(id = 'main', onReady = null, props = null) {
    const existingStatus = GlobalStatusProvider.get(id);

    if (existingStatus) {
      if (props) {
        existingStatus.add(props);
      }

      if (typeof onReady === 'function') {
        onReady(existingStatus);
      }

      return existingStatus;
    }

    const newStatus = GlobalStatusProvider.create(id, props);

    if (onReady) {
      newStatus.addOnReady(newStatus, onReady);
    }

    if (id !== 'main') {
      warn(`No <GlobalStatus ${`id="${id}"`} /> found.`);
    }

    return newStatus;
  }

  static get(id = 'main') {
    return GlobalStatusProvider.providers[id] || null;
  }

  static remove(id = 'main') {
    if (GlobalStatusProvider.providers[id]) {
      delete GlobalStatusProvider.providers[id];
    }
  }

  static prepareItemWithStatusId(item, status_id = null) {
    if (typeof item === 'string') {
      item = {
        text: item
      };
    }

    if (!item.status_id) {
      item.status_id = status_id && status_id !== 'status-main' ? status_id : slugify(JSON.stringify(item));
    }

    return item;
  }

  static combineMessages(stack) {
    const globalStatus = stack.reduce((acc, _cur) => {
      const cur = _extends({}, _cur);

      if (typeof cur.items === 'string' && cur.items[0] === '[') {
        cur.items = JSON.parse(cur.items);
      }

      if (cur.item) {
        if (typeof cur.item === 'string' && cur.item[0] === '{') {
          cur.item = JSON.parse(cur.item);
        }

        cur.items = cur.items || [];
        cur.items.push(cur.item);
      }

      if (cur.items) {
        cur.items = cur.items.reduce((acc, item) => {
          item = GlobalStatusProvider.prepareItemWithStatusId(item);
          const foundAtIndex = acc.findIndex(({
            status_id
          }) => status_id === item.status_id);

          if (foundAtIndex > -1) {
            acc[foundAtIndex] = item;
          } else {
            acc.push(item);
          }

          return acc;
        }, acc.items || []);
      }

      _extends(acc, cur);

      return acc;
    }, {});
    return globalStatus;
  }

}

GlobalStatusProvider.providers = {};

GlobalStatusProvider.create = (id = 'main', props = null) => {
  return GlobalStatusProvider.providers[id] = new GlobalStatusProviderItem(id, props);
};

class GlobalStatusProviderItem {
  constructor(id, props = null) {
    this.stack = [];
    this.globalStatus = {};
    this._onUpdateEvents = [];
    this._onReadyEvents = [];
    this.internal_id = id;

    if (props) {
      this.add(props);
    }
  }

  onUpdate(event) {
    if (this._onUpdateEvents.filter(cb => cb === event).length === 0) {
      this._onUpdateEvents.push(event);
    }
  }

  forceRerender(globalStatus, props, {
    buffer_delay = 0,
    isEmpty = false
  } = {}) {
    const run = () => {
      this._onUpdateEvents.forEach(event => {
        if (typeof event === 'function') {
          event(globalStatus, props, {
            isEmpty
          });
        }
      });
    };

    if (buffer_delay > 0) {
      clearTimeout(this._bufferDelayId);
      this._bufferDelayId = setTimeout(run, buffer_delay);
    } else {
      run();
    }
  }

  init(props) {
    return this.add(props, {
      preventRerender: true
    });
  }

  add(props, opts = {}) {
    this.remove('internal-close', {
      preventRerender: true
    });

    const newProps = _extends({}, props);

    if (!newProps.status_id) {
      newProps.status_id = makeUniqueId();
    }

    if (typeof newProps.show === 'undefined') {
      newProps.show = true;
    }

    if (newProps.children) {
      newProps.text = newProps.children;
      delete newProps.children;
    }

    const stackIndex = this.stack.findIndex(cur => cur.status_id === newProps.status_id);

    if (stackIndex > -1) {
      this.stack[stackIndex] = newProps;
    } else {
      this.stack.push(newProps);
    }

    const globalStatus = GlobalStatusProvider.combineMessages(this.stack);

    if (!(opts !== null && opts !== void 0 && opts.preventRerender)) {
      this.forceRerender(globalStatus, props, {
        buffer_delay: (props === null || props === void 0 ? void 0 : props.buffer_delay) > -1 ? props.buffer_delay : 0
      });
    }

    return globalStatus;
  }

  get(status_id) {
    return this.stack.find(cur => cur.status_id === status_id);
  }

  update(status_id, newProps, opts = {}) {
    if (status_id) {
      const item = this.get(status_id);

      if (!item) {
        this.add(newProps);
      }
    }

    this.stack = this.stack.map((cur, i, arr) => {
      if (!status_id ? i === arr.length - 1 : cur.status_id === status_id) {
        if (!status_id) {
          newProps = _extends({}, newProps);
          delete newProps.status_id;
        }

        return _extends({}, cur, newProps);
      }

      return cur;
    });

    if (!(opts !== null && opts !== void 0 && opts.preventRestack)) {
      this.restack(status_id);
    }

    const globalStatus = GlobalStatusProvider.combineMessages(this.stack);

    if (!(opts !== null && opts !== void 0 && opts.preventRerender)) {
      var _newProps;

      this.forceRerender(globalStatus, null, {
        buffer_delay: ((_newProps = newProps) === null || _newProps === void 0 ? void 0 : _newProps.buffer_delay) > -1 ? newProps.buffer_delay : 0
      });
    }
  }

  restack(status_id) {
    const item = this.get(status_id);

    if (item) {
      this.stack = this.stack.filter(cur => {
        return cur.status_id !== status_id;
      });
      this.stack.push(item);
    }
  }

  remove(status_id, opts = {}) {
    if (status_id) {
      this.stack = this.stack.filter(cur => {
        return cur.status_id !== status_id;
      });
      const globalStatus = GlobalStatusProvider.combineMessages(this.stack);

      if (!(opts !== null && opts !== void 0 && opts.preventRerender)) {
        this.forceRerender(globalStatus, null, {
          buffer_delay: (opts === null || opts === void 0 ? void 0 : opts.buffer_delay) > -1 ? opts.buffer_delay : 10
        });
      }
    }
  }

  empty() {
    this._onUpdateEvents.forEach((cb, i) => {
      this._onUpdateEvents[i] = null;
    });

    this._onUpdateEvents = [];

    this._onReadyEvents.forEach((cb, i) => {
      this._onReadyEvents[i] = null;
    });

    this._onReadyEvents = [];
  }

  unbind() {
    this.empty();
    GlobalStatusProvider.remove(this.internal_id);
  }

  isReady() {
    this._onReadyEvents = this._onReadyEvents.filter(({
      status,
      cb
    }, i) => {
      if (typeof cb === 'function') {
        cb(status);
      }

      this._onReadyEvents[i] = null;
      return false;
    });
    return true;
  }

  addOnReady(status, cb) {
    this._onReadyEvents.push({
      status,
      cb
    });
  }

}

if (typeof window !== 'undefined') {
  window.GlobalStatusProvider = GlobalStatusProvider;
}

export default GlobalStatusProvider;

const slugify = s => s.toLowerCase().replace(/[^\w\s-]/g, '').replace(/[\s_-]+/g, '-').replace(/^-+|-+$/g, '');