import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import keycode from 'keycode';
import { disableBodyScroll, enableBodyScroll, clearAllBodyScrollLocks } from '../../shared/libs/bodyScrollLock';
import { warn, isTrue, makeUniqueId, InteractionInvalidation, extendPropsWithContext, combineLabelledBy, combineDescribedBy, validateDOMAttributes, dispatchCustomElementEvent } from '../../shared/component-helper';
import Button from '../button/Button';
import ScrollView from '../../fragments/scroll-view/ScrollView';
import Context from '../../shared/Context';
export default class ModalContent extends React.PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      triggeredBy: null,
      triggeredByEvent: null
    };

    this._androidFocusHelper = () => {
      clearTimeout(this._androidFocusTimeout);
      this._androidFocusTimeout = setTimeout(() => {
        try {
          if (document.activeElement.tagName == 'INPUT' || document.activeElement.tagName == 'TEXTAREA') {
            document.activeElement.scrollIntoView();
          }
        } catch (e) {}
      }, 100);
    };

    this.preventClick = e => {
      if (e) {
        e.stopPropagation();
      }
    };

    this.onCloseClickHandler = e => {
      this.closeModal(e, {
        triggeredBy: 'button'
      });
    };

    this.onContentClickHandler = e => {
      this.closeModal(e, {
        triggeredBy: 'overlay',
        ifIsLatest: false
      });
    };

    this.onKeyDownHandler = e => {
      switch (keycode(e)) {
        case 'esc':
          {
            const mostCurrent = [...getListOfModalRoots()].pop();

            if (mostCurrent === this) {
              e.preventDefault();
              this.closeModal(e, {
                triggeredBy: 'keyboard'
              });
            }

            break;
          }
      }
    };

    this._contentRef = React.createRef();
    this._id = makeUniqueId();
    this._ii = new InteractionInvalidation();

    this._ii.setBypassSelector(['.dnb-modal__content', `#dnb-modal-${props.root_id || 'root'}`]);
  }

  componentDidMount() {
    this.addToIndex();
    const modalRoots = getListOfModalRoots();
    const firstLevel = modalRoots[0];

    if (firstLevel === this) {
      this.removeScrollPossibility();

      this._ii.activate();
    } else {
      modalRoots.forEach(modal => {
        if (modal !== this && typeof modal._iiLocal === 'undefined' && typeof modal._contentRef !== 'undefined') {
          modal._iiLocal = new InteractionInvalidation();

          modal._iiLocal.activate(modal._contentRef.current);
        }
      });
    }

    this.setAndroidFocusHelper();
    this.setFocus();
    const id = this.props.id;
    dispatchCustomElementEvent(this, 'on_open', {
      id
    });

    if (typeof document !== 'undefined') {
      document.addEventListener('keydown', this.onKeyDownHandler);
    }
  }

  componentWillUnmount() {
    clearTimeout(this._focusTimeout);
    const modalRoots = getListOfModalRoots();
    const firstLevel = modalRoots[0];
    this.removeFromIndex();

    if (firstLevel === this) {
      this._ii.revert();

      this.revertScrollPossibility();
    } else {
      try {
        const modal = modalRoots[modalRoots.length - 2];

        if (modal !== this && modal._iiLocal) {
          modal._iiLocal.revert();

          delete modal._iiLocal;
        }
      } catch (e) {
        warn(e);
      }
    }

    this.removeAndroidFocusHelper();
    const id = this.props.id;
    const {
      triggeredBy,
      triggeredByEvent
    } = this.state;
    dispatchCustomElementEvent(this, 'on_close', {
      id,
      event: triggeredByEvent,
      triggeredBy: triggeredBy || 'unmount'
    });

    if (typeof document !== 'undefined') {
      document.removeEventListener('keydown', this.onKeyDownHandler);
    }
  }

  setAndroidFocusHelper() {
    if (typeof window !== 'undefined' && typeof navigator !== 'undefined' && /Android/.test(navigator.appVersion)) {
      window.addEventListener('resize', this._androidFocusHelper);
    }
  }

  removeAndroidFocusHelper() {
    window.removeEventListener('resize', this._androidFocusHelper);
    clearTimeout(this._androidFocusTimeout);
  }

  addToIndex() {
    if (typeof window !== 'undefined') {
      try {
        window.__modalStack = window.__modalStack || [];

        window.__modalStack.push(this);
      } catch (e) {
        warn(e);
      }
    }
  }

  removeFromIndex() {
    if (typeof window !== 'undefined') {
      try {
        window.__modalStack = window.__modalStack || [];
        window.__modalStack = window.__modalStack.filter(cur => cur !== this);

        if (!window.__modalStack.length) {
          delete window.__modalStack;
        }
      } catch (e) {
        warn(e);
      }
    }
  }

  setFocus() {
    const {
      focus_selector,
      animation_duration
    } = this.props;
    const elem = this._contentRef.current;

    if (elem) {
      clearTimeout(this._focusTimeout);
      this._focusTimeout = setTimeout(() => {
        try {
          let focusElement;

          if (typeof focus_selector === 'string') {
            focusElement = elem.querySelector(focus_selector);
          } else if (focus_selector === false) {
            return;
          }

          if (!focusElement) {
            focusElement = elem.querySelector('h1:first-of-type, h2:first-of-type, .dnb-modal__close-button:not([disabled])');
          }

          if (!focusElement) {
            focusElement = elem;
          }

          if (focusElement) {
            focusElement.focus();
          }
        } catch (e) {
          warn(e);
        }
      }, parseFloat(animation_duration));
    }
  }

  removeScrollPossibility() {
    if (this._contentRef.current) {
      disableBodyScroll(this._contentRef.current);
    }
  }

  revertScrollPossibility() {
    enableBodyScroll(this._contentRef.current);
    clearAllBodyScrollLocks();
  }

  closeModal(event, _ref) {
    var _event$persist;

    let {
      triggeredBy
    } = _ref,
        params = _objectWithoutPropertiesLoose(_ref, ["triggeredBy"]);

    event === null || event === void 0 ? void 0 : (_event$persist = event.persist) === null || _event$persist === void 0 ? void 0 : _event$persist.call(event);
    this.setState({
      triggeredBy,
      triggeredByEvent: event
    }, () => {
      this.props.closeModal(event, _extends({
        triggeredBy
      }, params));
    });
  }

  render() {
    const _this$props = this.props,
          {
      mode,
      hide,
      title,
      labelled_by,
      modal_content,
      close_title,
      hide_close_button,
      spacing,
      prevent_core_style,
      no_animation,
      no_animation_on_mobile,
      min_width,
      max_width,
      fullscreen,
      align_content,
      container_placement,
      className,
      class: _className,
      content_class,
      overlay_class,
      content_id
    } = _this$props,
          rest = _objectWithoutPropertiesLoose(_this$props, ["mode", "hide", "title", "labelled_by", "modal_content", "close_title", "hide_close_button", "spacing", "prevent_close", "open_delay", "prevent_core_style", "animation_duration", "no_animation", "no_animation_on_mobile", "min_width", "max_width", "fullscreen", "align_content", "container_placement", "closeModal", "className", "class", "content_class", "overlay_class", "content_id", "toggleOpenClose", "children"]);

    const id = content_id || this._id;
    let minWidth = min_width;
    let maxWidth = max_width;

    if (minWidth && !maxWidth && parseFloat(minWidth) > 0) {
      maxWidth = 0;
    } else if (maxWidth && !minWidth && parseFloat(maxWidth) > 0) {
      minWidth = 0;
    }

    const contentParams = {
      role: 'main',
      'aria-modal': 'true',
      'aria-labelledby': combineLabelledBy(this.props, title ? id + '-title' : null, labelled_by),
      'aria-describedby': combineDescribedBy(this.props, id + '-content'),
      className: classnames('dnb-modal__content', (container_placement || mode === 'drawer') && `dnb-modal__content--${container_placement || 'right'}`, isTrue(fullscreen) ? 'dnb-modal__content--fullscreen' : fullscreen === 'auto' && 'dnb-modal__content--auto-fullscreen', content_class, mode && `dnb-modal__content--${mode}`, isTrue(hide) && 'dnb-modal__content--hide', isTrue(spacing) && 'dnb-modal__content--spacing', align_content && `dnb-modal__content__align--${align_content}`, isTrue(no_animation) && 'dnb-modal__content--no-animation', isTrue(no_animation_on_mobile) && 'dnb-modal__content--no-animation-on-mobile'),
      onClick: this.onContentClickHandler
    };

    const innerParams = _extends({
      className: classnames('dnb-modal__content__inner', className, _className, !isTrue(prevent_core_style) && 'dnb-core-style'),
      style: (minWidth || maxWidth) && {
        minWidth,
        maxWidth
      },
      onClick: this.preventClick,
      onTouchStart: this.preventClick,
      onKeyDown: this.onKeyDownHandler
    }, rest);

    const spacingParams = {
      tabIndex: -1,
      className: "dnb-modal__content__spacing dnb-no-focus"
    };
    const overlayParams = {
      className: classnames('dnb-modal__overlay', overlay_class, hide && 'dnb-modal__overlay--hide', mode && `dnb-modal__overlay--${mode}`, isTrue(no_animation) && 'dnb-modal__overlay--no-animation', isTrue(no_animation_on_mobile) && 'dnb-modal__overlay--no-animation-on-mobile')
    };
    validateDOMAttributes(this.props, innerParams);
    return React.createElement(React.Fragment, null, React.createElement("div", _extends({
      id: id
    }, contentParams), React.createElement(ScrollView, innerParams, React.createElement("div", _extends({}, spacingParams, {
      ref: this._contentRef
    }), title && React.createElement("h1", {
      id: id + '-title',
      className: "dnb-modal__title " + (mode === 'drawer' ? 'dnb-h--x-large' : 'dnb-h--large')
    }, title), !isTrue(hide_close_button) && React.createElement(CloseButton, {
      on_click: this.onCloseClickHandler,
      close_title: close_title
    }), React.createElement("div", {
      id: id + '-content',
      className: "dnb-modal__wrapper"
    }, modal_content)))), React.createElement("span", _extends({}, overlayParams, {
      "aria-hidden": "true"
    })));
  }

}
ModalContent.defaultProps = {
  mode: null,
  hide: null,
  id: null,
  root_id: null,
  labelled_by: null,
  focus_selector: null,
  content_id: null,
  title: null,
  close_title: null,
  hide_close_button: null,
  spacing: null,
  prevent_close: null,
  prevent_core_style: null,
  animation_duration: null,
  no_animation: null,
  no_animation_on_mobile: null,
  min_width: null,
  max_width: null,
  fullscreen: null,
  align_content: null,
  container_placement: null,
  class: null,
  overlay_class: null,
  content_class: null,
  closeModal: null,
  className: null,
  children: null
};
process.env.NODE_ENV !== "production" ? ModalContent.propTypes = {
  modal_content: PropTypes.node.isRequired,
  mode: PropTypes.oneOf(['modal', 'drawer']),
  hide: PropTypes.bool,
  id: PropTypes.string,
  root_id: PropTypes.string,
  labelled_by: PropTypes.string,
  focus_selector: PropTypes.string,
  content_id: PropTypes.string,
  title: PropTypes.node,
  close_title: PropTypes.string,
  hide_close_button: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  spacing: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  prevent_close: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  prevent_core_style: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  animation_duration: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  no_animation: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  no_animation_on_mobile: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  min_width: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  max_width: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  fullscreen: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  align_content: PropTypes.string,
  container_placement: PropTypes.string,
  class: PropTypes.string,
  content_class: PropTypes.string,
  overlay_class: PropTypes.string,
  closeModal: PropTypes.func.isRequired,
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.node, PropTypes.func])
} : void 0;
export class CloseButton extends React.PureComponent {
  render() {
    const _extendPropsWithConte = extendPropsWithContext(this.props, CloseButton.defaultProps, this.context.getTranslation(this.props).Modal, this.context.FormRow, this.context.ModalContent),
          {
      on_click,
      style_type,
      close_title,
      className = null
    } = _extendPropsWithConte,
          rest = _objectWithoutPropertiesLoose(_extendPropsWithConte, ["on_click", "style_type", "close_title", "className"]);

    if (style_type === 'cross') {
      rest.icon_size = 'medium';
    } else {
      rest.size = 'large';
      rest.icon_size = 'basis';
      rest.text = close_title;
    }

    return React.createElement(Button, _extends({
      type: "button",
      variant: "secondary",
      className: classnames('dnb-modal__close-button', className, style_type && `dnb-modal__close-button--${style_type}`),
      icon: "close",
      on_click: on_click
    }, rest));
  }

}
CloseButton.contextType = Context;
CloseButton.defaultProps = {
  style_type: null,
  close_title: null,
  className: null
};
process.env.NODE_ENV !== "production" ? CloseButton.propTypes = {
  style_type: PropTypes.oneOf(['button', 'cross']),
  on_click: PropTypes.func.isRequired,
  close_title: PropTypes.string,
  className: PropTypes.string
} : void 0;
export function getListOfModalRoots(index = null) {
  if (typeof window !== 'undefined') {
    try {
      const stack = window.__modalStack || [];

      if (index !== null) {
        if (index === -1 && stack.length) {
          return stack[stack.length - 1];
        } else if (index > -1) {
          return stack[index];
        }
      }

      return stack;
    } catch (e) {
      warn(e);
    }
  }

  return [];
}