import _extends from "@babel/runtime/helpers/esm/extends";
import React from 'react';
import PropTypes from 'prop-types';
import Context from '../../shared/Context';
import { isTrue, dispatchCustomElementEvent } from '../../shared/component-helper';
import { ContentObject } from './PaginationHelpers';
import PaginationContext from './PaginationContext';
export default class PaginationProvider extends React.PureComponent {
  static getDerivedStateFromProps(props, state) {
    if (state._listenForPropChanges) {
      if (props.page_count !== null) {
        state.pageCount = parseFloat(props.page_count) || 1;
      }

      if (props.current_page !== null) {
        state.currentPage = parseFloat(props.current_page) || 1;
      }

      if (typeof state.startupPage === 'undefined') {
        state.startupPage = parseFloat(props.startup_page) || parseFloat(props.current_page) || 1;
      }

      state.parallelLoadCount = parseFloat(props.parallel_load_count) || 1;
      state.minTime = parseFloat(props.min_wait_time) || 0;
      state.placeMakerBeforeContent = isTrue(props.place_maker_before_content);

      if (props.useMarkerOnly) {
        if (!state.lowerPage) {
          state.lowerPage = state.upperPage = state.startupPage;
        }
      }

      if (props.reset_pagination_handler !== null && isTrue(props.reset_pagination_handler)) {
        if (props.useMarkerOnly) {
          state.lowerPage = state.upperPage = state.startupPage;
        }
      }

      if (props.reset_content_handler !== null && isTrue(props.reset_content_handler)) {
        state.items = [];
        state.pageCount = parseFloat(props.page_count) || 1;
      }

      if (typeof props.items === 'string' && props.items[0] === '[') {
        state.items = JSON.parse(props.items);
      } else if (Array.isArray(props.items)) {
        state.items = props.items;
      }
    }

    state._listenForPropChanges = true;
    return state;
  }

  constructor(_props) {
    super(_props);

    this.setContent = (newContent, content = null, position = null) => {
      if (!Array.isArray(newContent) && content) {
        newContent = [newContent, content];
      }

      const pageNo = parseFloat(newContent[0]);
      newContent = newContent[1];

      if (typeof newContent === 'function') {
        content = newContent();
      } else if (React.isValidElement(newContent)) {
        content = newContent;
      }

      if (content) {
        let itemToPrepare = this.state.items.find(({
          pageNo: p
        }) => p === pageNo);
        let items = null;

        if (!itemToPrepare) {
          items = this.prefillItems(pageNo, {
            position,
            skipObserver: true
          });
          itemToPrepare = items.find(({
            pageNo: p
          }) => p === pageNo);
        }

        if (itemToPrepare.content) {
          itemToPrepare.update(content);
        } else {
          itemToPrepare.insert(content);
        }

        this.setState({
          items: [...(items || this.state.items)],
          currentPage: pageNo,
          _listenForPropChanges: false
        }, this.callOnPageUpdate);
      }
    };

    this.resetContent = () => {
      clearTimeout(this.resetContentTimeout);
      this.resetContentTimeout = setTimeout(() => {
        this.setState({
          items: [],
          _listenForPropChanges: false
        });
      }, 10);
    };

    this.resetInfinity = (pageNo = this.state.startupPage) => {
      const lowerPage = pageNo;
      const upperPage = pageNo;
      const currentPage = pageNo;
      this._hasEndedInfinity = true;
      this.setState({
        hasEndedInfinity: true,
        lowerPage,
        upperPage,
        currentPage,
        _listenForPropChanges: false
      });
      clearTimeout(this.resetInfinityTimeout);
      this.resetInfinityTimeout = setTimeout(() => {
        this.startInfinity();
      }, this.state.minTime);
    };

    this.startInfinity = () => {
      this._hasEndedInfinity = false;
      this.setState({
        hasEndedInfinity: false,
        _listenForPropChanges: false
      });
    };

    this.endInfinity = () => {
      this._hasEndedInfinity = true;
      this.setState({
        hasEndedInfinity: true,
        _listenForPropChanges: false
      }, () => {
        const pageNo = this.state.currentPage + 1;
        dispatchCustomElementEvent(this, 'on_end', _extends({
          page: pageNo,
          pageNo
        }, this));
      });
    };

    this.setItems = (items, cb) => {
      this.setState({
        items,
        _listenForPropChanges: false
      }, cb);
    };

    this.prefillItems = (pageNo, props = {}, items = this.state.items) => {
      const position = props.position || (pageNo < this.state.currentPage ? 'before' : 'after');

      const obj = _extends({
        pageNo,
        position,
        skipObserver: false
      }, props);

      switch (position) {
        case 'before':
          return [new ContentObject(obj), ...items];

        case 'after':
          return [...items, new ContentObject(obj)];
      }
    };

    this.setStateHandler = (state, cb) => {
      this.setState(_extends({}, state, {
        _listenForPropChanges: false
      }), cb);
    };

    this.callOnPageUpdate = () => {
      if (Array.isArray(this._updateStack)) {
        this._updateStack.forEach(cb => {
          if (typeof cb === 'function') {
            cb();
          }
        });

        this._updateStack = [];
      }
    };

    this.onPageUpdate = fn => {
      this._updateStack = this._updateStack || [];

      this._updateStack.push(fn);
    };

    this.state = {
      items: [],
      isLoading: false,
      _listenForPropChanges: true
    };

    if (_props.rerender) {
      this.rerender = ({
        current: store
      }) => {
        if (store && store.pageNo > 0) {
          clearTimeout(this.rerenderTimeout);
          this.rerenderTimeout = setTimeout(() => this.setContent(store.pageNo, store.content), 1);
        }
      };

      _props.rerender.current = this.rerender;
    }
  }

  componentDidMount() {
    const {
      set_content_handler,
      reset_content_handler,
      reset_pagination_handler,
      end_infinity_handler
    } = this.props;

    if (typeof set_content_handler === 'function') {
      set_content_handler(this.setContent);
    }

    if (typeof reset_content_handler === 'function') {
      reset_content_handler(this.resetContent);
    }

    if (typeof reset_pagination_handler === 'function') {
      reset_pagination_handler(this.resetInfinity);
    }

    if (typeof end_infinity_handler === 'function') {
      end_infinity_handler(this.endInfinity);
    }

    if (this.props.store && this.props.store.current) {
      const store = this.props.store.current;
      this.setContent(store.pageNo, store.content);
    }

    this._isMounted = true;
  }

  componentWillUnmount() {
    clearTimeout(this.rerenderTimeout);
    clearTimeout(this.resetContentTimeout);
    clearTimeout(this.resetInfinityTimeout);
    clearTimeout(this.callOnPageUpdateTimeout);
    this._isMounted = false;
  }

  render() {
    const {
      children
    } = this.props;

    if (this.props.useMarkerOnly) {
      clearTimeout(this.callOnPageUpdateTimeout);
      this.callOnPageUpdateTimeout = setTimeout(this.callOnPageUpdate, 1);
    }

    return React.createElement(PaginationContext.Provider, {
      value: _extends({}, this.context, {
        pagination: _extends({
          setContent: this.setContent,
          resetContent: this.resetContent,
          resetInfinity: this.resetInfinity,
          resetPagination: this.resetInfinity,
          endInfinity: this.endInfinity,
          setItems: this.setItems,
          prefillItems: this.prefillItems,
          setState: this.setStateHandler,
          onPageUpdate: this.onPageUpdate,
          _hasEndedInfinity: this._hasEndedInfinity
        }, this.props, this.state)
      })
    }, children);
  }

}
PaginationProvider.contextType = Context;
PaginationProvider.defaultProps = {
  startup_page: null,
  current_page: null,
  page_count: null,
  set_content_handler: null,
  reset_content_handler: null,
  reset_pagination_handler: null,
  end_infinity_handler: null,
  rerender: null,
  store: null,
  useMarkerOnly: null,
  children: null
};
process.env.NODE_ENV !== "production" ? PaginationProvider.propTypes = {
  startup_page: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  current_page: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  page_count: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  set_content_handler: PropTypes.func,
  reset_content_handler: PropTypes.func,
  reset_pagination_handler: PropTypes.func,
  end_infinity_handler: PropTypes.func,
  rerender: PropTypes.shape({
    current: PropTypes.func
  }),
  store: PropTypes.shape({
    current: PropTypes.oneOfType([PropTypes.object, PropTypes.func])
  }),
  useMarkerOnly: PropTypes.bool,
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node, PropTypes.object, PropTypes.array])
} : void 0;