import _extends from "@babel/runtime/helpers/esm/extends";
import React from 'react';
import PropTypes from 'prop-types';
import keycode from 'keycode';
import Context from '../../shared/Context';
import { warn, isTrue, roundToNearest, isInsideScrollView, detectOutsideClick, dispatchCustomElementEvent } from '../../shared/component-helper';
import { getOffsetTop, getOffsetLeft, hasSelectedText } from '../../shared/helpers';
import { getData, normalizeData, findClosest, getSelectedItemValue, parseContentTitle, getEventData, prepareStartupState, prepareDerivedState } from './DrawerListHelpers';
import DrawerListContext from './DrawerListContext';
import { disableBodyScroll, enableBodyScroll } from '../../shared/libs/bodyScrollLock';
export default class DrawerListProvider extends React.PureComponent {
  static getDerivedStateFromProps(props, state) {
    return prepareDerivedState(props, state);
  }

  constructor(props) {
    super(props);

    this.enableBodyLock = () => {
      if (this._refUl.current) {
        this._bodyLockIsEnabled = true;
        disableBodyScroll(this._refUl.current);
      }
    };

    this.disableBodyLock = () => {
      if (this._bodyLockIsEnabled && this._refUl.current) {
        this._bodyLockIsEnabled = null;
        enableBodyScroll(this._refUl.current);
      }
    };

    this.scrollToItem = (active_item, {
      scrollTo = true
    } = {}) => {
      clearTimeout(this._scrollTimeout);
      this._scrollTimeout = setTimeout(() => {
        if (this._refUl.current && parseFloat(active_item) > -1) {
          try {
            const ulElement = this._refUl.current;
            const liElement = this.getActiveElement();

            if (liElement) {
              const top = liElement.offsetTop;

              if (ulElement.scrollTo) {
                if (scrollTo === false || window.IS_TEST) {
                  ulElement.style.scrollBehavior = 'auto';
                }

                ulElement.scrollTo({
                  top,
                  behavior: scrollTo ? 'smooth' : 'auto'
                });

                if (scrollTo === false) {
                  ulElement.style.scrollBehavior = 'smooth';
                }
              } else if (ulElement.scrollTop) {
                ulElement.scrollTop = top;
              }

              if (!isTrue(this.props.prevent_focus) && liElement) {
                liElement.focus();
                dispatchCustomElementEvent(this, 'on_show_focus', {
                  element: liElement
                });
              }
            }
          } catch (e) {
            warn('List could not scroll into element:', e);
          }
        }
      }, 1);
    };

    this.scrollToAndSetActiveItem = (active_item, {
      fireSelectEvent = false,
      scrollTo = true,
      event = null
    } = {}) => {
      if (parseFloat(active_item) > -1) {
        this.setState({
          active_item,
          _listenForPropChanges: false
        }, () => {
          const {
            selected_item
          } = this.state;

          if (fireSelectEvent) {
            const attributes = this.attributes;
            const ret = dispatchCustomElementEvent(this.state, 'on_select', {
              active_item,
              value: getSelectedItemValue(selected_item, this.state),
              data: getEventData(active_item, this.state.data),
              event,
              attributes
            });

            if (ret === false) {
              return;
            }
          }

          this.scrollToItem(active_item, {
            scrollTo
          });
        });
      } else if (!isTrue(this.props.prevent_focus)) {
        if (this._refUl.current) {
          this._refUl.current.focus({
            preventScroll: true
          });

          dispatchCustomElementEvent(this, 'on_show_focus', {
            element: this._refUl.current
          });
        }
      }
    };

    this.setWrapperElement = (wrapper_element = this.props.wrapper_element) => {
      if (typeof wrapper_element === 'string' && typeof document !== 'undefined') {
        wrapper_element = document.querySelector(wrapper_element);
      }

      if (wrapper_element) {
        this.setState({
          wrapper_element,
          _listenForPropChanges: false
        });
      }

      return this;
    };

    this.anchorKeyDownHandler = e => {
      const key = keycode(e);

      switch (key) {
        case 'tab':
          try {
            const nextEl = this.meta.shift ? e.target.previousSibling : e.target.nextSibling;
            e.stopPropagation();
            e.preventDefault();

            if (nextEl) {
              this.focusAnchorElem(nextEl);
            } else {
              this.getActiveElement().focus();
            }
          } catch (e) {}

          break;

        case 'enter':
        case 'space':
          e.stopPropagation();
          break;

        default:
          break;
      }
    };

    this.setMetaKey = e => {
      const we = typeof window !== 'undefined' && window.event ? window.event : e;
      this.meta = {
        cmd: we.metaKey,
        ctrl: we.ctrlKey,
        shift: we.shiftKey,
        alt: we.altKey
      };
    };

    this.onKeyUpHandler = e => {
      this.setMetaKey(e);
    };

    this.onKeyDownHandler = e => {
      const key = keycode(e);

      if (/command|alt|shift|ctrl/.test(key)) {
        this.setMetaKey(e);
      }

      if (isTrue(this.props.prevent_close)) {
          let isSameDrawer = false;

          try {
            isSameDrawer = typeof document !== 'undefined' && `${document.activeElement.getAttribute('id')}-ul` === this._refUl.current.getAttribute('id');
          } catch (e) {
            warn(e);
          }

          if (!isSameDrawer || key === 'tab') {
            return;
          }
        }

      if (isTrue(this.state.ignore_events) && key !== 'tab') {
        return;
      }

      if (!this.state.isOpen) {
        return;
      }

      let active_item = parseFloat(this.state.active_item);

      if (isNaN(active_item)) {
        active_item = -1;
      }

      const total = this.state.data && this.state.data.length - 1;

      switch (key) {
        case 'up':
          {
            e.preventDefault();
            active_item = this.getPrevActiveItem();

            if (isNaN(active_item)) {
              active_item = this.getFirstItem() || 0;
            }
          }
          break;

        case 'down':
          {
            e.preventDefault();
            const activeItem = this.getCurrentActiveItem();

            if (active_item === -1 || isNaN(activeItem)) {
              active_item = this.getFirstItem() || 0;
            } else {
              active_item = this.getNextActiveItem();
            }
          }
          break;

        case 'page up':
        case 'home':
          {
            e.preventDefault();
            active_item = this.getFirstItem() || 0;
          }
          break;

        case 'page down':
        case 'end':
          {
            e.preventDefault();
            active_item = this.getLastItem();

            if (isNaN(active_item)) {
              active_item = total;
            }
          }
          break;

        case 'enter':
        case 'space':
          {
            active_item = this.getCurrentActiveItem();

            if (isTrue(this.props.skip_keysearch) ? active_item > -1 && key !== 'space' : true) {
              e.preventDefault();
              this.selectItem(active_item, {
                fireSelectEvent: true,
                event: e
              });
            }
          }
          break;

        case 'esc':
          {
            this.setHidden({
              event: e
            });
            e.preventDefault();
          }
          break;

        case 'tab':
          {
            if (active_item > -1) {
              const anchorElem = this.getAnchorElem();

              if (anchorElem) {
                e.preventDefault();
                this.focusAnchorElem(anchorElem);
                return;
              }
            }

            this.setHidden({
              event: e
            });
          }
          break;

        default:
          active_item = this.findItemByValue(keycode(e));
          break;
      }

      if (parseFloat(active_item) > -1 && active_item !== this.state.active_item) {
        this.scrollToAndSetActiveItem(active_item, {
          fireSelectEvent: true,
          event: e
        });
      }
    };

    this.getSelectedElement = () => {
      var _this$_refUl$current;

      return ((_this$_refUl$current = this._refUl.current) === null || _this$_refUl$current === void 0 ? void 0 : _this$_refUl$current.querySelector('li.dnb-drawer-list__option--selected')) || this._refUl.current || {
        getAttribute: () => null
      };
    };

    this.getCurrentSelectedItem = () => {
      const elem = this.getSelectedElement();
      return parseFloat(elem && elem.getAttribute('data-item'));
    };

    this.getActiveElement = () => {
      var _this$_refUl$current2;

      return ((_this$_refUl$current2 = this._refUl.current) === null || _this$_refUl$current2 === void 0 ? void 0 : _this$_refUl$current2.querySelector('li.dnb-drawer-list__option--focus')) || this.getSelectedElement();
    };

    this.getCurrentActiveItem = () => {
      const elem = this.getActiveElement();
      return parseFloat(elem && elem.getAttribute('data-item'));
    };

    this.getNextActiveItem = () => {
      const elem = this.getActiveElement().nextSibling;
      return parseFloat(elem && elem.getAttribute('data-item'));
    };

    this.getPrevActiveItem = () => {
      const elem = this.getActiveElement().previousSibling;
      return parseFloat(elem && elem.getAttribute('data-item'));
    };

    this.getFirstItem = () => {
      var _this$_refUl$current3;

      const elem = (_this$_refUl$current3 = this._refUl.current) === null || _this$_refUl$current3 === void 0 ? void 0 : _this$_refUl$current3.querySelector('li.dnb-drawer-list__option.first-of-type');
      return parseFloat(elem && elem.getAttribute('data-item'));
    };

    this.getLastItem = () => {
      var _this$_refUl$current4;

      const elem = (_this$_refUl$current4 = this._refUl.current) === null || _this$_refUl$current4 === void 0 ? void 0 : _this$_refUl$current4.querySelector('li.dnb-drawer-list__option.last-of-type');
      return parseFloat(elem && elem.getAttribute('data-item'));
    };

    this.setOutsideClickObserver = () => {
      this.removeOutsideClickObserver();
      this.outsideClick = detectOutsideClick([this.state.wrapper_element, this._refRoot.current, this._refUl.current], this.setHidden);

      if (typeof document !== 'undefined') {
        document.addEventListener('keydown', this.onKeyDownHandler);
        document.addEventListener('keyup', this.onKeyUpHandler);
      }
    };

    this.assignObservers = () => {
      this.setDirectionObserver();
      this.setScrollObserver();
      this.setOutsideClickObserver();
    };

    this.setVisible = () => {
      clearTimeout(this._hideTimeout);

      if (this.state.opened && this.state.hidden === false) {
        return;
      }

      this.searchCache = null;
      this.setState({
        hidden: false,
        opened: true,
        _listenForPropChanges: false
      }, this.assignObservers);

      const delayHandler = () => {
        this.setState({
          isOpen: true,
          _listenForPropChanges: false
        });
      };

      if (isTrue(this.props.no_animation)) {
        delayHandler();
      } else {
        this._hideTimeout = setTimeout(delayHandler, DrawerListProvider.blurDelay);
      }

      const {
        selected_item,
        active_item
      } = this.state;
      dispatchCustomElementEvent(this.state, 'on_show', {
        data: getEventData(parseFloat(selected_item) > -1 ? selected_item : active_item, this.state.data),
        attributes: this.attributes,
        ulElement: this._refUl.current
      });
    };

    this.setHidden = (args = {}, onStateComplete = null) => {
      if (!this.state.opened || isTrue(this.props.prevent_close)) {
        if (typeof onStateComplete === 'function') {
          onStateComplete();
        }

        return;
      }

      clearTimeout(this._hideTimeout);
      const {
        selected_item,
        active_item
      } = this.state;
      const res = dispatchCustomElementEvent(this.state, 'on_hide', _extends({}, args, {
        data: getEventData(parseFloat(selected_item) > -1 ? selected_item : active_item, this.state.data),
        attributes: this.attributes
      }));

      if (res !== false) {
        this.setState({
          opened: false,
          _listenForPropChanges: false
        });

        const delayHandler = () => {
          this.setState({
            hidden: true,
            isOpen: false,
            _listenForPropChanges: false
          });

          if (typeof onStateComplete === 'function') {
            onStateComplete();
          }
        };

        if (isTrue(this.props.no_animation)) {
          delayHandler();
        } else {
          this._hideTimeout = setTimeout(delayHandler, DrawerListProvider.blurDelay);
        }

        this.waitUntilUlIsReady = false;
        this.removeDirectionObserver();
        this.removeScrollObserver();
        this.removeOutsideClickObserver();
      }
    };

    this.setDataHandler = (data, cb = null, {
      overwriteOriginalData = false
    } = {}) => {
      if (!data) {
        return;
      }

      if (typeof data === 'function') {
        data = getData(data);
      }

      data = normalizeData(data);
      this.setState({
        data,
        original_data: overwriteOriginalData ? data : this.state.original_data,
        _listenForPropChanges: false
      }, () => {
        this.refreshScrollObserver();
        typeof cb === 'function' && cb(data);
      });
      return this;
    };

    this.setStateHandler = (state, cb = null) => {
      this.setState(_extends({}, state, {
        _listenForPropChanges: false
      }), cb);
      return this;
    };

    this.selectItem = (itemToSelect, {
      fireSelectEvent = false,
      event = null
    } = {}) => {
      if (event && typeof event.persist === 'function') {
        event.persist();
      }

      if (itemToSelect === -1) {
        itemToSelect = null;
      }

      const data = getEventData(itemToSelect, this.state.data) || null;
      const value = getSelectedItemValue(itemToSelect, this.state);
      const attributes = this.attributes;
      const attr = {
        selected_item: itemToSelect,
        value,
        data,
        event,
        attributes
      };
      const res = dispatchCustomElementEvent(this.state, 'on_pre_change', attr);

      if (res === false || hasSelectedText()) {
        return;
      }

      const {
        keep_open,
        no_animation,
        prevent_selection
      } = this.props;
      const doCallOnChange = this.state.selected_item !== itemToSelect;

      const onSelectionIsComplete = () => {
        const delayHandler = () => {
          if (doCallOnChange) {
            dispatchCustomElementEvent(this.state, 'on_change', attr);
          }

          if (fireSelectEvent) {
            dispatchCustomElementEvent(this.state, 'on_select', _extends({}, attr, {
              active_item: itemToSelect
            }));
          }

          if (!isTrue(keep_open)) {
            this.setHidden();
          }
        };

        if (isTrue(no_animation)) {
          delayHandler();
        } else {
          clearTimeout(this._selectTimeout);
          this._selectTimeout = setTimeout(delayHandler, DrawerListProvider.blurDelay / 2);
        }
      };

      if (isTrue(prevent_selection)) {
        onSelectionIsComplete();
      } else {
        this.setState({
          _listenForPropChanges: false,
          selected_item: itemToSelect,
          active_item: itemToSelect
        }, onSelectionIsComplete);
      }
    };

    this.attributes = {};
    this.state = _extends({
      tagName: 'dnb-drawer-list',
      cache_hash: '',
      active_item: null,
      selected_item: null,
      ignore_events: false
    }, prepareStartupState(props), {
      _listenForPropChanges: true
    });
    this._refRoot = React.createRef();
    this._refShell = React.createRef();
    this._refUl = React.createRef();
    this._refTriangle = React.createRef();
  }

  componentDidMount() {
    if (isTrue(this.props.opened)) {
      this.setVisible();
    }
  }

  componentDidUpdate(prevProps) {
    if (this.props.opened !== null && this.props.opened !== prevProps.opened) {
      if (isTrue(this.props.opened)) {
        this.setVisible();
      } else if (isTrue(this.props.opened) === false) {
        this.setHidden();
      }
    }
  }

  componentWillUnmount() {
    clearInterval(this._outsideClickTimeout);
    clearTimeout(this._hideTimeout);
    clearTimeout(this._selectTimeout);
    clearTimeout(this._scrollTimeout);
    clearTimeout(this._ddTimeout);
    clearTimeout(this._doTimeout);
    this.setState({
      opened: false,
      _listenForPropChanges: false
    });
    this.removeDirectionObserver();
    this.removeScrollObserver();
    this.removeOutsideClickObserver();
  }

  refreshScrollObserver() {
    if (typeof window === 'undefined' || !this._refUl.current) {
      return;
    }

    this.itemSpots = this.state.data.reduce((acc, cur, i) => {
      var _this$_refUl$current5;

      const element = (_this$_refUl$current5 = this._refUl.current) === null || _this$_refUl$current5 === void 0 ? void 0 : _this$_refUl$current5.querySelector(`li.dnb-drawer-list__option:nth-of-type(${i + 1})`);

      if (element) {
        acc[element.offsetTop] = {
          i
        };
      }

      return acc;
    }, {});
    this.itemSpotsCount = Object.keys(this.itemSpots).length;
  }

  setScrollObserver() {
    if (!isTrue(this.props.enable_closest_observer) || typeof window === 'undefined' || !this._refUl.current) {
      return;
    }

    this.removeScrollObserver();
    this.itemSpotsCount = 1;

    try {
      let closestToTop = null,
          closestToBottom = null,
          tmpToTop,
          tmpToBottom;

      this.setOnScroll = () => {
        if (!this._refUl.current) {
          return;
        }

        if (this.itemSpotsCount <= 1) {
          this.refreshScrollObserver();
        }

        const counts = Object.keys(this.itemSpots);
        closestToBottom = findClosest(counts, this._refUl.current.scrollTop + this._refUl.current.offsetHeight);
        closestToTop = findClosest(counts, this._refUl.current.scrollTop);

        if (this.itemSpots[closestToTop] && closestToTop !== tmpToTop) {
          this.setState({
            closestToTop: this.itemSpots[closestToTop].i,
            _listenForPropChanges: false
          });
        }

        if (closestToBottom !== tmpToBottom && this.itemSpots[closestToBottom]) {
          this.setState({
            closestToBottom: this.itemSpots[closestToBottom].i,
            _listenForPropChanges: false
          });
        }

        tmpToTop = closestToTop;
        tmpToBottom = closestToBottom;
      };

      this._refUl.current.addEventListener('scroll', this.setOnScroll);

      this.setOnScroll();
    } catch (e) {
      warn('List could not set onScroll:', e);
    }
  }

  removeScrollObserver() {
    if (typeof window !== 'undefined' && this.setOnScroll) {
      window.removeEventListener('resize', this.setOnScroll);
      this.setOnScroll = null;
    }
  }

  setDirectionObserver() {
    if (typeof window === 'undefined' || typeof document === 'undefined' || !(this.state.wrapper_element || this._refRoot.current)) {
      return;
    }

    const {
      enable_body_lock,
      use_drawer_on_mobile,
      scrollable,
      min_height,
      max_height,
      on_resize,
      page_offset,
      observer_element
    } = this.props;
    const useBodyLock = isTrue(enable_body_lock);
    const useDrawer = isTrue(use_drawer_on_mobile);
    const isScrollable = isTrue(scrollable);
    const customMinHeight = parseFloat(min_height) * 16;
    const customMaxHeight = parseFloat(max_height) || 0;
    let customElem = typeof observer_element === 'string' ? document.querySelector(observer_element) : null;

    if (!customElem) {
      customElem = isInsideScrollView(this._refRoot.current, true);
    }

    this.removeDirectionObserver();
    const directionOffset = 96;
    const spaceToTopOffset = 2 * 16;
    const spaceToBottomOffset = 2 * 16;
    const elem = this.state.wrapper_element || this._refRoot.current;

    const renderDirection = () => {
      try {
        const rootElem = customElem || document.documentElement;
        const pageYOffset = !isNaN(parseFloat(page_offset)) ? parseFloat(page_offset) : rootElem.scrollTop;
        const spaceToTop = getOffsetTop(elem) + elem.offsetHeight - pageYOffset;
        const spaceToBottom = rootElem.clientHeight - (getOffsetTop(elem) + elem.offsetHeight) + pageYOffset;
        const direction = Math.max(spaceToBottom - directionOffset, directionOffset) < customMinHeight && spaceToTop > customMinHeight ? 'top' : 'bottom';
        let max_height = customMaxHeight;

        if (!(max_height > 0)) {
          max_height = direction === 'top' ? spaceToTop - ((this.state.wrapper_element || this._refRoot.current).offsetHeight || 0) - spaceToTopOffset : spaceToBottom - spaceToBottomOffset;
          let vh = 0;

          if (typeof window.visualViewport !== 'undefined') {
            vh = window.visualViewport.height;
          } else {
            vh = Math.max(document.documentElement.clientHeight, window.innerHeight || 0);
          }

          vh = vh * (isScrollable ? 0.7 : 0.9);

          if (max_height > vh) {
            max_height = vh;
          }

          max_height = roundToNearest(max_height, 8) / 16;
        }

        if (this.props.direction === 'auto') {
          this.setState({
            direction,
            _listenForPropChanges: false
          });
        }

        this.setState({
          max_height,
          _listenForPropChanges: false
        });

        if (on_resize) {
          dispatchCustomElementEvent(this.state, 'on_resize', {
            direction,
            max_height
          });
        }
      } catch (e) {
        warn('List could not set onResize:', e);
      }
    };

    this.setDirection = e => {
      clearTimeout(this._ddTimeout);
      this._ddTimeout = setTimeout(renderDirection, 30);

      if (useDrawer && e.type === 'resize') {
        if (!this._bodyLockIsEnabled && (window.innerWidth / 16 <= 40 || window.innerHeight / 16 <= 40)) {
          this.enableBodyLock();
        } else if (this._bodyLockIsEnabled && !useBodyLock) {
          this.disableBodyLock();
        }
      }

      if (e.type === 'resize') {
        this.correctHiddenView();
      }
    };

    this._rootElem = customElem || window;

    this._rootElem.addEventListener('scroll', this.setDirection);

    if (typeof window.visualViewport !== 'undefined') {
      window.visualViewport.addEventListener('scroll', this.setDirection);
      window.visualViewport.addEventListener('resize', this.setDirection);
    } else {
      window.addEventListener('resize', this.setDirection);
    }

    clearTimeout(this._doTimeout);
    this._doTimeout = setTimeout(() => {
      if (useBodyLock || useDrawer && (window.innerWidth / 16 <= 40 || window.innerHeight / 16 <= 40)) {
        this.enableBodyLock();
      }

      this.correctHiddenView();
      this.refreshScrollObserver();
      const {
        selected_item,
        active_item
      } = this.state;
      this.scrollToAndSetActiveItem(parseFloat(active_item) > -1 ? active_item : selected_item, {
        scrollTo: false
      });
    }, 1);
    renderDirection();
  }

  correctHiddenView() {
    try {
      const ui = this._refUl.current;
      const spaceToLeft = getOffsetLeft(ui);
      const spaceToRight = window.innerWidth - (getOffsetLeft(ui) + ui.offsetWidth);
      const tri = this._refTriangle.current.style;
      const shell = this._refShell.current.style;

      if (spaceToLeft < 0) {
        shell.transform = `translateX(${Math.abs(spaceToLeft)}px)`;
        tri.right = `${Math.abs(spaceToLeft)}px`;
      } else if (spaceToRight < 0) {
        shell.transform = `translateX(${spaceToRight}px)`;
        tri.left = `${Math.abs(spaceToRight)}px`;
      } else {
        if (shell.transform) {
          shell.transform = '';
          tri.left = 'auto';
          tri.right = 'auto';
        }
      }
    } catch (e) {}
  }

  findItemByValue(value) {
    if (isTrue(this.props.skip_keysearch)) {
      return;
    }

    let index = -1;

    try {
      value = String(value).toLowerCase();

      if (this.changedOrderFor !== value) {
        this.searchCache = null;
        this.changedOrderFor = null;
      }

      this.searchCache = this.searchCache || this.state.data.reduce((acc, itemData, i) => {
        const str = String(parseContentTitle(itemData, {
          separator: ' ',
          removeNumericOnlyValues: true
        }));
        const firstLetter = String(str[0]).toLowerCase();
        acc[firstLetter] = acc[firstLetter] || [];
        acc[firstLetter].push({
          i
        });
        return acc;
      }, {});
      const found = this.searchCache[value];
      index = found && found[0] && found[0].i > -1 ? found[0].i : -1;

      if (found && found.length > 1) {
        found.push(found.shift());
        this.changedOrderFor = value;
      }
    } catch (e) {
      warn('List could not findItemByValue:', e);
    }

    return index;
  }

  removeDirectionObserver() {
    this.disableBodyLock();
    clearTimeout(this._ddTimeout);

    if (typeof window !== 'undefined' && this.setDirection) {
      var _this$_rootElem;

      (_this$_rootElem = this._rootElem) === null || _this$_rootElem === void 0 ? void 0 : _this$_rootElem.removeEventListener('scroll', this.setDirection);

      if (typeof window.visualViewport !== 'undefined') {
        window.visualViewport.removeEventListener('scroll', this.setDirection);
        window.visualViewport.removeEventListener('resize', this.setDirection);
      } else {
        window.removeEventListener('resize', this.setDirection);
      }

      this.setDirection = null;
    }
  }

  getAnchorElem() {
    try {
      return this.getActiveElement().querySelector('a:not(:focus):first-of-type');
    } catch (e) {
      return null;
    }
  }

  focusAnchorElem(elem) {
    if (elem) {
      elem.focus({
        preventScroll: true
      });

      if (!elem._hkh) {
        elem._hkh = true;
        elem.addEventListener('keydown', this.anchorKeyDownHandler);
      }
    }
  }

  removeOutsideClickObserver() {
    if (this.outsideClick) {
      this.outsideClick.remove();
    }

    if (typeof document !== 'undefined') {
      document.removeEventListener('keydown', this.onKeyDownHandler);
      document.removeEventListener('keyup', this.onKeyUpHandler);
    }
  }

  render() {
    return React.createElement(DrawerListContext.Provider, {
      value: _extends({}, this.context, {
        drawerList: _extends({
          attributes: this.attributes,
          _refRoot: this._refRoot,
          _refShell: this._refShell,
          _refUl: this._refUl,
          _refTriangle: this._refTriangle,
          _rootElem: this._rootElem,
          getActiveElement: this.getActiveElement,
          setData: this.setDataHandler,
          setState: this.setStateHandler,
          setWrapperElement: this.setWrapperElement,
          assignObservers: this.assignObservers,
          setVisible: this.setVisible,
          setHidden: this.setHidden,
          selectItem: this.selectItem,
          scrollToItem: this.scrollToItem,
          scrollToAndSetActiveItem: this.scrollToAndSetActiveItem
        }, this.state)
      })
    }, this.props.children);
  }

}
DrawerListProvider.contextType = Context;
DrawerListProvider.defaultProps = {
  no_animation: false,
  prevent_selection: false,
  direction: 'auto',
  wrapper_element: null,
  prevent_close: false,
  keep_open: false,
  prevent_focus: false,
  skip_keysearch: false,
  use_drawer_on_mobile: null,
  enable_body_lock: null,
  page_offset: null,
  observer_element: null,
  enable_closest_observer: null,
  opened: null,
  scrollable: null,
  min_height: 10,
  max_height: null,
  on_resize: null,
  children: null
};
DrawerListProvider.blurDelay = 201;
process.env.NODE_ENV !== "production" ? DrawerListProvider.propTypes = {
  no_animation: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  prevent_selection: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  direction: PropTypes.oneOf(['auto', 'top', 'bottom']),
  wrapper_element: PropTypes.oneOfType([PropTypes.object, PropTypes.func, PropTypes.node]),
  prevent_close: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  keep_open: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  prevent_focus: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  skip_keysearch: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  use_drawer_on_mobile: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  enable_body_lock: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  page_offset: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  observer_element: PropTypes.oneOfType([PropTypes.string, PropTypes.node]),
  enable_closest_observer: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  opened: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  scrollable: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  min_height: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  max_height: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  on_resize: PropTypes.func,
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node, PropTypes.object, PropTypes.array])
} : void 0;